@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -77,25 +77,21 @@
 // #define QAX_SUPPORT_BORDERSPACE
 
 // missing interface from win32api
-#if defined(Q_CC_GNU)
-#   if !defined(IOleInPlaceObjectWindowless)
-#       undef INTERFACE
-#       define INTERFACE IOleInPlaceObjectWindowless
-        DECLARE_INTERFACE_(IOleInPlaceObjectWindowless,IOleInPlaceObject)
-        {
-           STDMETHOD(QueryInterface)(THIS_ REFIID,PVOID*) PURE;
-           STDMETHOD_(ULONG,AddRef)(THIS) PURE;
-           STDMETHOD_(ULONG,Release)(THIS) PURE;
-           STDMETHOD(GetWindow)(THIS_ HWND*) PURE;
-           STDMETHOD(ContextSensitiveHelp)(THIS_ BOOL) PURE;
-           STDMETHOD(InPlaceDeactivate)(THIS) PURE;
-           STDMETHOD(UIDeactivate)(THIS) PURE;
-           STDMETHOD(SetObjectRects)(THIS_ LPCRECT,LPCRECT) PURE;
-           STDMETHOD(ReactivateAndUndo)(THIS) PURE;
-           STDMETHOD(OnWindowMessage)(THIS_ UINT, WPARAM, LPARAM, LRESULT*) PURE;
-           STDMETHOD(GetDropTarget)(THIS_ IDropTarget**) PURE;
-        };
-#   endif
+#if defined(Q_CC_GNU) && !defined(__MINGW64_VERSION_MAJOR)
+    DECLARE_INTERFACE_(IOleInPlaceObjectWindowless,IOleInPlaceObject)
+    {
+       STDMETHOD(QueryInterface)(THIS_ REFIID,PVOID*) PURE;
+       STDMETHOD_(ULONG,AddRef)(THIS) PURE;
+       STDMETHOD_(ULONG,Release)(THIS) PURE;
+       STDMETHOD(GetWindow)(THIS_ HWND*) PURE;
+       STDMETHOD(ContextSensitiveHelp)(THIS_ BOOL) PURE;
+       STDMETHOD(InPlaceDeactivate)(THIS) PURE;
+       STDMETHOD(UIDeactivate)(THIS) PURE;
+       STDMETHOD(SetObjectRects)(THIS_ LPCRECT,LPCRECT) PURE;
+       STDMETHOD(ReactivateAndUndo)(THIS) PURE;
+       STDMETHOD(OnWindowMessage)(THIS_ UINT, WPARAM, LPARAM, LRESULT*) PURE;
+       STDMETHOD(GetDropTarget)(THIS_ IDropTarget**) PURE;
+    };
 #endif
 
 #include "../shared/qaxtypes.h"
@@ -1012,7 +1008,7 @@ HRESULT WINAPI QAxClientSite::TranslateAccelerator(LPMSG lpMsg, DWORD /*grfModif
         }
     }
     // ActiveQt based in-processes-servers will handle the event properly, so
-    // we dont need to send this key event to the host.
+    // we don't need to send this key event to the host.
     return S_OK;
 }
 
@@ -1438,9 +1434,6 @@ extern Q_GUI_EXPORT bool qt_win_ignoreNextMouseReleaseEvent;
 
 HRESULT WINAPI QAxClientSite::EnableModeless(BOOL fEnable)
 {
-#if !defined(Q_OS_WINCE)
-    LockWindowUpdate(host->window()->winId());
-#endif
     EnableWindow(host->window()->winId(), fEnable);
 
     if (!fEnable) {
@@ -1451,9 +1444,6 @@ HRESULT WINAPI QAxClientSite::EnableModeless(BOOL fEnable)
             QApplicationPrivate::leaveModal(host);
     }
     qt_win_ignoreNextMouseReleaseEvent = false;
-#if !defined(Q_OS_WINCE)
-    LockWindowUpdate(0);
-#endif
     return S_OK;
 }
 
