@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -94,6 +94,7 @@ QEglContext::QEglContext()
     , current(false)
     , ownsContext(true)
     , sharing(false)
+    , apiChanged(false)
 {
     QEglContextTracker::ref();
 }
@@ -210,7 +211,7 @@ EGLConfig QEgl::defaultConfig(int devType, API api, ConfigOptions options)
     else
         configId = qgetenv("QT_GL_EGL_CONFIG");
     if (!configId.isEmpty()) {
-        // Overriden, so get the EGLConfig for the specified config ID:
+        // Overridden, so get the EGLConfig for the specified config ID:
         EGLint properties[] = {
             EGL_CONFIG_ID, (EGLint)configId.toInt(),
             EGL_NONE
@@ -267,7 +268,7 @@ EGLConfig QEgl::defaultConfig(int devType, API api, ConfigOptions options)
         configAttribs.setValue(EGL_STENCIL_SIZE, 1);
         configAttribs.setValue(EGL_SAMPLE_BUFFERS, 1);
 #ifndef QT_OPENGL_ES_2
-        // Aditionally, the GL1 engine likes to have a depth buffer for clipping
+        // Additionally, the GL1 engine likes to have a depth buffer for clipping
         configAttribs.setValue(EGL_DEPTH_SIZE, 1);
 #endif
     }
@@ -435,9 +436,20 @@ bool QEglContext::makeCurrent(EGLSurface surface)
         return false;
     }
 
+#ifdef Q_OS_SYMBIAN
+    apiChanged = false;
+    if (currentContext(apiType)
+            && currentContext(apiType)->ctx != eglGetCurrentContext()) {
+        // some other EGL based API active. Complete its rendering
+        eglWaitClient();
+        apiChanged = true;
+    }
+#endif
+
     // If lazyDoneCurrent() was called on the surface, then we may be able
     // to assume that it is still current within the thread.
-    if (surface == currentSurface && currentContext(apiType) == this) {
+    if (surface == currentSurface && currentContext(apiType) == this
+            && !apiChanged) {
         current = true;
         return true;
     }
@@ -512,6 +524,13 @@ bool QEglContext::swapBuffers(EGLSurface surface)
     bool ok = eglSwapBuffers(QEgl::display(), surface);
     if (!ok)
         qWarning() << "QEglContext::swapBuffers():" << QEgl::errorString();
+
+#ifdef Q_OS_SYMBIAN
+    if (apiChanged) {
+        eglWaitClient();
+        apiChanged = false;
+    }
+#endif
     return ok;
 }
 
