@@ -1,40 +1,40 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 ** This file is part of the Qt Linguist of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
 ** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this
+** file. Please review the following information to ensure the GNU Lesser
+** General Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** Alternatively, this file may be used under the terms of the GNU General
+** Public License version 3.0 as published by the Free Software Foundation
+** and appearing in the file LICENSE.GPL included in the packaging of this
+** file. Please review the following information to ensure the GNU General
+** Public License version 3.0 requirements will be met:
+** http://www.gnu.org/copyleft/gpl.html.
+**
+** Other Usage
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+**
+**
+**
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -50,11 +50,18 @@
 #include <QtCore/QString>
 #include <QtCore/QTextCodec>
 #include <QtCore/QTextStream>
+#include <QtCore/QCoreApplication>
+
+#include <iostream>
 
 #include <ctype.h>              // for isXXX()
 
 QT_BEGIN_NAMESPACE
 
+class LU {
+    Q_DECLARE_TR_FUNCTIONS(LUpdate)
+};
+
 /* qmake ignore Q_OBJECT */
 
 static QString MagicComment(QLatin1String("TRANSLATOR"));
@@ -65,47 +72,44 @@ static QString MagicComment(QLatin1String("TRANSLATOR"));
 
 class HashString {
 public:
-    HashString() : m_hashed(false) {}
-    explicit HashString(const QString &str) : m_str(str), m_hashed(false) {}
-    void setValue(const QString &str) { m_str = str; m_hashed = false; }
+    HashString() : m_hash(0x80000000) {}
+    explicit HashString(const QString &str) : m_str(str), m_hash(0x80000000) {}
+    void setValue(const QString &str) { m_str = str; m_hash = 0x80000000; }
     const QString &value() const { return m_str; }
     bool operator==(const HashString &other) const { return m_str == other.m_str; }
 private:
     QString m_str;
+    // qHash() of a QString is only 28 bits wide, so we can use
+    // the highest bit(s) as the "hash valid" flag.
     mutable uint m_hash;
-    mutable bool m_hashed;
     friend uint qHash(const HashString &str);
 };
 
 uint qHash(const HashString &str)
 {
-    if (!str.m_hashed) {
-        str.m_hashed = true;
+    if (str.m_hash & 0x80000000)
         str.m_hash = qHash(str.m_str);
-    }
     return str.m_hash;
 }
 
 class HashStringList {
 public:
-    explicit HashStringList(const QList<HashString> &list) : m_list(list), m_hashed(false) {}
+    explicit HashStringList(const QList<HashString> &list) : m_list(list), m_hash(0x80000000) {}
     const QList<HashString> &value() const { return m_list; }
     bool operator==(const HashStringList &other) const { return m_list == other.m_list; }
 private:
     QList<HashString> m_list;
     mutable uint m_hash;
-    mutable bool m_hashed;
     friend uint qHash(const HashStringList &list);
 };
 
 uint qHash(const HashStringList &list)
 {
-    if (!list.m_hashed) {
-        list.m_hashed = true;
+    if (list.m_hash & 0x80000000) {
         uint hash = 0;
         foreach (const HashString &qs, list.m_list) {
-            hash ^= qHash(qs) ^ 0xa09df22f;
-            hash = (hash << 13) | (hash >> 19);
+            hash ^= qHash(qs) ^ 0x0ad9f526;
+            hash = ((hash << 13) & 0x0fffffff) | (hash >> 15);
         }
         list.m_hash = hash;
     }
@@ -215,18 +219,22 @@ public:
 private:
     struct IfdefState {
         IfdefState() {}
-        IfdefState(int _braceDepth, int _parenDepth) :
+        IfdefState(int _bracketDepth, int _braceDepth, int _parenDepth) :
+            bracketDepth(_bracketDepth),
             braceDepth(_braceDepth),
             parenDepth(_parenDepth),
             elseLine(-1)
         {}
 
         SavedState state;
+        int bracketDepth, bracketDepth1st;
         int braceDepth, braceDepth1st;
         int parenDepth, parenDepth1st;
         int elseLine;
     };
 
+    std::ostream &yyMsg(int line = 0);
+
     uint getChar();
     uint getToken();
     bool getMacroArgs();
@@ -259,6 +267,8 @@ private:
     static QStringList stringListifySegments(const QList<HashString> &namespaces);
     bool qualifyOneCallbackOwn(const Namespace *ns, void *context) const;
     bool qualifyOneCallbackUsing(const Namespace *ns, void *context) const;
+    bool qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
+                    NamespaceList *resolved, QSet<HashStringList> *visitedUsings) const;
     bool qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
                     NamespaceList *resolved) const;
     bool fullyQualify(const NamespaceList &namespaces, int nsCnt,
@@ -278,14 +288,14 @@ private:
 
     enum {
         Tok_Eof, Tok_class, Tok_friend, Tok_namespace, Tok_using, Tok_return,
-        Tok_tr = 10, Tok_trUtf8, Tok_translate, Tok_translateUtf8, Tok_trid,
-        Tok_Q_OBJECT = 20, Tok_Q_DECLARE_TR_FUNCTIONS,
+        Tok_tr, Tok_trUtf8, Tok_translate, Tok_translateUtf8, Tok_trid,
+        Tok_Q_OBJECT, Tok_Q_DECLARE_TR_FUNCTIONS,
         Tok_Ident, Tok_Comment, Tok_String, Tok_Arrow, Tok_Colon, Tok_ColonColon,
-        Tok_Equals,
-        Tok_LeftBrace = 30, Tok_RightBrace, Tok_LeftParen, Tok_RightParen, Tok_Comma, Tok_Semicolon,
-        Tok_Null = 40, Tok_Integer,
-        Tok_QuotedInclude = 50, Tok_AngledInclude,
-        Tok_Other = 99
+        Tok_Equals, Tok_LeftBracket, Tok_RightBracket,
+        Tok_LeftBrace, Tok_RightBrace, Tok_LeftParen, Tok_RightParen, Tok_Comma, Tok_Semicolon,
+        Tok_Null, Tok_Integer,
+        Tok_QuotedInclude, Tok_AngledInclude,
+        Tok_Other
     };
 
     // Tokenizer state
@@ -297,16 +307,17 @@ private:
     QString yyWord;
     qlonglong yyInteger;
     QStack<IfdefState> yyIfdefStack;
+    int yyBracketDepth;
     int yyBraceDepth;
     int yyParenDepth;
     int yyLineNo;
     int yyCurLineNo;
+    int yyBracketLineNo;
     int yyBraceLineNo;
     int yyParenLineNo;
 
     // the string to read from and current position in the string
     QTextCodec *yySourceCodec;
-    bool yySourceIsUnicode;
     QString yyInStr;
     const ushort *yyInPtr;
 
@@ -338,9 +349,11 @@ CppParser::CppParser(ParseResults *_results)
         results = new ParseResults;
         directInclude = false;
     }
+    yyBracketDepth = 0;
     yyBraceDepth = 0;
     yyParenDepth = 0;
     yyCurLineNo = 1;
+    yyBracketLineNo = 1;
     yyBraceLineNo = 1;
     yyParenLineNo = 1;
     yyAtNewline = true;
@@ -348,12 +361,17 @@ CppParser::CppParser(ParseResults *_results)
     inDefine = false;
 }
 
+
+std::ostream &CppParser::yyMsg(int line)
+{
+    return std::cerr << qPrintable(yyFileName) << ':' << (line ? line : yyLineNo) << ": ";
+}
+
 void CppParser::setInput(const QString &in)
 {
     yyInStr = in;
     yyFileName = QString();
     yySourceCodec = 0;
-    yySourceIsUnicode = true;
     yyForceUtf8 = true;
 }
 
@@ -362,7 +380,6 @@ void CppParser::setInput(QTextStream &ts, const QString &fileName)
     yyInStr = ts.readAll();
     yyFileName = fileName;
     yySourceCodec = ts.codec();
-    yySourceIsUnicode = yySourceCodec->name().startsWith("UTF-");
     yyForceUtf8 = false;
 }
 
@@ -472,6 +489,7 @@ STRING(class);
 STRING(findMessage);
 STRING(friend);
 STRING(namespace);
+STRING(operator);
 STRING(qtTrId);
 STRING(return);
 STRING(struct);
@@ -569,7 +587,7 @@ uint CppParser::getToken()
                 yyCh = getChar();
                 if (yyCh == 'f') {
                     // if, ifdef, ifndef
-                    yyIfdefStack.push(IfdefState(yyBraceDepth, yyParenDepth));
+                    yyIfdefStack.push(IfdefState(yyBracketDepth, yyBraceDepth, yyParenDepth));
                     yyCh = getChar();
                 } else if (yyCh == 'n') {
                     // include
@@ -608,16 +626,20 @@ uint CppParser::getToken()
                     if (!yyIfdefStack.isEmpty()) {
                         IfdefState &is = yyIfdefStack.top();
                         if (is.elseLine != -1) {
-                            if (yyBraceDepth != is.braceDepth1st || yyParenDepth != is.parenDepth1st)
-                                qWarning("%s:%d: Parenthesis/brace mismatch between "
-                                         "#if and #else branches; using #if branch\n",
-                                         qPrintable(yyFileName), is.elseLine);
+                            if (yyBracketDepth != is.bracketDepth1st
+                                || yyBraceDepth != is.braceDepth1st
+                                || yyParenDepth != is.parenDepth1st)
+                                yyMsg(is.elseLine)
+                                    << qPrintable(LU::tr("Parenthesis/bracket/brace mismatch between "
+                                                         "#if and #else branches; using #if branch\n"));
                         } else {
+                            is.bracketDepth1st = yyBracketDepth;
                             is.braceDepth1st = yyBraceDepth;
                             is.parenDepth1st = yyParenDepth;
                             saveState(&is.state);
                         }
                         is.elseLine = yyLineNo;
+                        yyBracketDepth = is.bracketDepth;
                         yyBraceDepth = is.braceDepth;
                         yyParenDepth = is.parenDepth;
                     }
@@ -627,10 +649,13 @@ uint CppParser::getToken()
                     if (!yyIfdefStack.isEmpty()) {
                         IfdefState is = yyIfdefStack.pop();
                         if (is.elseLine != -1) {
-                            if (yyBraceDepth != is.braceDepth1st || yyParenDepth != is.parenDepth1st)
-                                qWarning("%s:%d: Parenthesis/brace mismatch between "
-                                         "#if and #else branches; using #if branch\n",
-                                         qPrintable(yyFileName), is.elseLine);
+                            if (yyBracketDepth != is.bracketDepth1st
+                                || yyBraceDepth != is.braceDepth1st
+                                || yyParenDepth != is.parenDepth1st)
+                                yyMsg(is.elseLine)
+                                    << qPrintable(LU::tr("Parenthesis/brace mismatch between "
+                                                         "#if and #else branches; using #if branch\n"));
+                            yyBracketDepth = is.bracketDepth1st;
                             yyBraceDepth = is.braceDepth1st;
                             yyParenDepth = is.parenDepth1st;
                             loadState(&is.state);
@@ -655,8 +680,7 @@ uint CppParser::getToken()
                         forever {
                             yyCh = getChar();
                             if (yyCh == EOF) {
-                                qWarning("%s:%d: Unterminated C++ comment\n",
-                                         qPrintable(yyFileName), yyLineNo);
+                                yyMsg() << qPrintable(LU::tr("Unterminated C++ comment\n"));
                                 break;
                             }
 
@@ -730,6 +754,20 @@ uint CppParser::getToken()
                 if (yyWord == strnamespace)
                     return Tok_namespace;
                 break;
+            case 'o':
+                if (yyWord == stroperator) {
+                    // Operator overload declaration/definition.
+                    // We need to prevent those characters from confusing the followup
+                    // parsing. Actually using them does not add value, so just eat them.
+                    while (isspace(yyCh))
+                       yyCh = getChar();
+                    while (yyCh == '+' || yyCh == '-' || yyCh == '*' || yyCh == '/' || yyCh == '%'
+                           || yyCh == '=' || yyCh == '<' || yyCh == '>' || yyCh == '!'
+                           || yyCh == '&' || yyCh == '|' || yyCh == '~' || yyCh == '^'
+                           || yyCh == '[' || yyCh == ']')
+                        yyCh = getChar();
+                }
+                break;
             case 'q':
                 if (yyWord == strqtTrId)
                     return Tok_trid;
@@ -786,8 +824,7 @@ uint CppParser::getToken()
                     forever {
                         yyCh = getChar();
                         if (yyCh == EOF) {
-                            qWarning("%s:%d: Unterminated C++ comment\n",
-                                     qPrintable(yyFileName), yyLineNo);
+                            yyMsg() << qPrintable(LU::tr("Unterminated C++ comment\n"));
                             break;
                         }
                         *ptr++ = yyCh;
@@ -820,8 +857,7 @@ uint CppParser::getToken()
                 yyWord.resize(ptr - (ushort *)yyWord.unicode());
 
                 if (yyCh != '"')
-                    qWarning("%s:%d: Unterminated C++ string\n",
-                              qPrintable(yyFileName), yyLineNo);
+                    yyMsg() << qPrintable(LU::tr("Unterminated C++ string\n"));
                 else
                     yyCh = getChar();
                 return Tok_String;
@@ -858,8 +894,7 @@ uint CppParser::getToken()
 
                 forever {
                     if (yyCh == EOF || yyCh == '\n') {
-                        qWarning("%s:%d: Unterminated C++ character\n",
-                                  qPrintable(yyFileName), yyLineNo);
+                        yyMsg() << "Unterminated C++ character\n";
                         break;
                     }
                     yyCh = getChar();
@@ -878,9 +913,9 @@ uint CppParser::getToken()
             case '}':
                 if (yyBraceDepth == yyMinBraceDepth) {
                     if (!inDefine)
-                        qWarning("%s:%d: Excess closing brace in C++ code"
-                                  " (or abuse of the C++ preprocessor)\n",
-                                  qPrintable(yyFileName), yyCurLineNo);
+                        yyMsg(yyCurLineNo)
+                            << qPrintable(LU::tr("Excess closing brace in C++ code"
+                                                 " (or abuse of the C++ preprocessor)\n"));
                     // Avoid things getting messed up even more
                     yyCh = getChar();
                     return Tok_Semicolon;
@@ -896,13 +931,28 @@ uint CppParser::getToken()
                 return Tok_LeftParen;
             case ')':
                 if (yyParenDepth == 0)
-                    qWarning("%s:%d: Excess closing parenthesis in C++ code"
-                             " (or abuse of the C++ preprocessor)\n",
-                             qPrintable(yyFileName), yyCurLineNo);
+                    yyMsg(yyCurLineNo)
+                        << qPrintable(LU::tr("Excess closing parenthesis in C++ code"
+                                             " (or abuse of the C++ preprocessor)\n"));
                 else
                     yyParenDepth--;
                 yyCh = getChar();
                 return Tok_RightParen;
+            case '[':
+                if (yyBracketDepth == 0)
+                    yyBracketLineNo = yyCurLineNo;
+                yyBracketDepth++;
+                yyCh = getChar();
+                return Tok_LeftBracket;
+            case ']':
+                if (yyBracketDepth == 0)
+                    yyMsg(yyCurLineNo)
+                        << qPrintable(LU::tr("Excess closing bracket in C++ code"
+                                             " (or abuse of the C++ preprocessor)\n"));
+                else
+                    yyBracketDepth--;
+                yyCh = getChar();
+                return Tok_RightBracket;
             case ',':
                 yyCh = getChar();
                 return Tok_Comma;
@@ -1039,15 +1089,16 @@ QStringList CppParser::stringListifySegments(const QList<HashString> &segments)
 }
 
 struct QualifyOneData {
-    QualifyOneData(const NamespaceList &ns, int nsc, const HashString &seg, NamespaceList *rslvd)
-        : namespaces(ns), nsCount(nsc), segment(seg), resolved(rslvd)
+    QualifyOneData(const NamespaceList &ns, int nsc, const HashString &seg, NamespaceList *rslvd,
+                   QSet<HashStringList> *visited)
+        : namespaces(ns), nsCount(nsc), segment(seg), resolved(rslvd), visitedUsings(visited)
     {}
 
     const NamespaceList &namespaces;
     int nsCount;
     const HashString &segment;
     NamespaceList *resolved;
-    QSet<HashStringList> visitedUsings;
+    QSet<HashStringList> *visitedUsings;
 };
 
 bool CppParser::qualifyOneCallbackOwn(const Namespace *ns, void *context) const
@@ -1081,18 +1132,19 @@ bool CppParser::qualifyOneCallbackUsing(const Namespace *ns, void *context) cons
 {
     QualifyOneData *data = (QualifyOneData *)context;
     foreach (const HashStringList &use, ns->usings)
-        if (!data->visitedUsings.contains(use)) {
-            data->visitedUsings.insert(use);
-            if (qualifyOne(use.value(), use.value().count(), data->segment, data->resolved))
+        if (!data->visitedUsings->contains(use)) {
+            data->visitedUsings->insert(use);
+            if (qualifyOne(use.value(), use.value().count(), data->segment, data->resolved,
+                           data->visitedUsings))
                 return true;
         }
     return false;
 }
 
 bool CppParser::qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
-                           NamespaceList *resolved) const
+                           NamespaceList *resolved, QSet<HashStringList> *visitedUsings) const
 {
-    QualifyOneData data(namespaces, nsCnt, segment, resolved);
+    QualifyOneData data(namespaces, nsCnt, segment, resolved, visitedUsings);
 
     if (visitNamespace(namespaces, nsCnt, &CppParser::qualifyOneCallbackOwn, &data))
         return true;
@@ -1100,6 +1152,14 @@ bool CppParser::qualifyOne(const NamespaceList &namespaces, int nsCnt, const Has
     return visitNamespace(namespaces, nsCnt, &CppParser::qualifyOneCallbackUsing, &data);
 }
 
+bool CppParser::qualifyOne(const NamespaceList &namespaces, int nsCnt, const HashString &segment,
+                           NamespaceList *resolved) const
+{
+    QSet<HashStringList> visitedUsings;
+
+    return qualifyOne(namespaces, nsCnt, segment, resolved, &visitedUsings);
+}
+
 bool CppParser::fullyQualify(const NamespaceList &namespaces, int nsCnt,
                              const QList<HashString> &segments, bool isDeclaration,
                              NamespaceList *resolved, QStringList *unresolved) const
@@ -1256,8 +1316,7 @@ void CppParser::processInclude(const QString &file, ConversionData &cd,
     QString cleanFile = QDir::cleanPath(file);
 
     if (inclusions.contains(cleanFile)) {
-        qWarning("%s:%d: circular inclusion of %s\n",
-                 qPrintable(yyFileName), yyLineNo, qPrintable(cleanFile));
+        yyMsg() << qPrintable(LU::tr("circular inclusion of %1\n").arg(cleanFile));
         return;
     }
 
@@ -1281,9 +1340,7 @@ void CppParser::processInclude(const QString &file, ConversionData &cd,
 
     QFile f(cleanFile);
     if (!f.open(QIODevice::ReadOnly)) {
-        qWarning("%s:%d: Cannot open %s: %s\n",
-                 qPrintable(yyFileName), yyLineNo,
-                 qPrintable(cleanFile), qPrintable(f.errorString()));
+        yyMsg() << qPrintable(LU::tr("Cannot open %1: %2\n").arg(cleanFile, f.errorString()));
         return;
     }
 
@@ -1430,24 +1487,24 @@ QString CppParser::transcode(const QString &str, bool utf8)
 {
     static const char tab[] = "abfnrtv";
     static const char backTab[] = "\a\b\f\n\r\t\v";
-    const QString in = (!utf8 || yySourceIsUnicode)
-        ? str : QString::fromUtf8(yySourceCodec->fromUnicode(str).data());
-    QString out;
+    // This function has to convert back to bytes, as C's \0* sequences work at that level.
+    const QByteArray in = yyForceUtf8 ? str.toUtf8() : tor->codec()->fromUnicode(str);
+    QByteArray out;
 
     out.reserve(in.length());
     for (int i = 0; i < in.length();) {
-        ushort c = in[i++].unicode();
+        uchar c = in[i++];
         if (c == '\\') {
             if (i >= in.length())
                 break;
-            c = in[i++].unicode();
+            c = in[i++];
 
             if (c == '\n')
                 continue;
 
             if (c == 'x') {
                 QByteArray hex;
-                while (i < in.length() && isxdigit((c = in[i].unicode()))) {
+                while (i < in.length() && isxdigit((c = in[i]))) {
                     hex += c;
                     i++;
                 }
@@ -1456,7 +1513,7 @@ QString CppParser::transcode(const QString &str, bool utf8)
                 QByteArray oct;
                 int n = 0;
                 oct += c;
-                while (n < 2 && i < in.length() && (c = in[i].unicode()) >= '0' && c < '8') {
+                while (n < 2 && i < in.length() && (c = in[i]) >= '0' && c < '8') {
                     i++;
                     n++;
                     oct += c;
@@ -1464,13 +1521,14 @@ QString CppParser::transcode(const QString &str, bool utf8)
                 out += oct.toUInt(0, 8);
             } else {
                 const char *p = strchr(tab, c);
-                out += QChar(QLatin1Char(!p ? c : backTab[p - tab]));
+                out += !p ? c : backTab[p - tab];
             }
         } else {
             out += c;
         }
     }
-    return out;
+    return (utf8 || yyForceUtf8) ? QString::fromUtf8(out.constData(), out.length())
+                                 : tor->codec()->toUnicode(out);
 }
 
 void CppParser::recordMessage(
@@ -1527,6 +1585,12 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
     yyCh = getChar();
     yyTok = getToken();
     while (yyTok != Tok_Eof) {
+        // these are array indexing operations. we ignore them entirely
+        // so they don't confuse our scoping of static initializers.
+        // we enter the loop by either reading a left bracket or by an
+        // #else popping the state.
+        while (yyBracketDepth)
+            yyTok = getToken();
         //qDebug() << "TOKEN: " << yyTok;
         switch (yyTok) {
         case Tok_QuotedInclude: {
@@ -1615,8 +1679,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                     // Forward-declared class definitions can be namespaced.
                     NamespaceList nsl;
                     if (!fullyQualify(namespaces, quali, true, &nsl, 0)) {
-                        qWarning("%s:%d: Ignoring definition of undeclared qualified class\n",
-                                 qPrintable(yyFileName), yyLineNo);
+                        yyMsg() << "Ignoring definition of undeclared qualified class\n";
                         break;
                     }
                     namespaceDepths.push(namespaces.count());
@@ -1630,6 +1693,8 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                 functionContextUnresolved.clear(); // Pointless
                 prospectiveContext.clear();
                 pendingContext.clear();
+
+                yyTok = getToken();
             }
             break;
         case Tok_namespace:
@@ -1643,6 +1708,11 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                 if (yyTok == Tok_LeftBrace) {
                     namespaceDepths.push(namespaces.count());
                     enterNamespace(&namespaces, ns);
+
+                    functionContext = namespaces;
+                    functionContextUnresolved.clear();
+                    prospectiveContext.clear();
+                    pendingContext.clear();
                     yyTok = getToken();
                 } else if (yyTok == Tok_Equals) {
                     // e.g. namespace Is = OuterSpace::InnerSpace;
@@ -1716,8 +1786,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
             if (!tor)
                 goto case_default;
             if (!sourcetext.isEmpty())
-                qWarning("%s:%d: //%% cannot be used with tr() / QT_TR_NOOP(). Ignoring\n",
-                         qPrintable(yyFileName), yyLineNo);
+                yyMsg() << qPrintable(LU::tr("//% cannot be used with tr() / QT_TR_NOOP(). Ignoring\n"));
             utf8 = (yyTok == Tok_trUtf8);
             line = yyLineNo;
             yyTok = getToken();
@@ -1734,14 +1803,12 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                         plural = true;
                     }
                 }
-                if (!pendingContext.isEmpty()) {
+                if (!pendingContext.isEmpty() && !prefix.startsWith(strColons)) {
                     QStringList unresolved;
                     if (!fullyQualify(namespaces, pendingContext, true, &functionContext, &unresolved)) {
                         functionContextUnresolved = unresolved.join(strColons);
-                        qWarning("%s:%d: Qualifying with unknown namespace/class %s::%s\n",
-                                 qPrintable(yyFileName), yyLineNo,
-                                 qPrintable(stringifyNamespace(functionContext)),
-                                 qPrintable(unresolved.first()));
+                        yyMsg() << qPrintable(LU::tr("Qualifying with unknown namespace/class %1::%2\n")
+                                              .arg(stringifyNamespace(functionContext)).arg(unresolved.first()));
                     }
                     pendingContext.clear();
                 }
@@ -1749,8 +1816,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                     if (functionContextUnresolved.isEmpty()) {
                         int idx = functionContext.length();
                         if (idx < 2) {
-                            qWarning("%s:%d: tr() cannot be called without context\n",
-                                     qPrintable(yyFileName), yyLineNo);
+                            yyMsg() << qPrintable(LU::tr("tr() cannot be called without context\n"));
                             break;
                         }
                         Namespace *fctx;
@@ -1759,9 +1825,8 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                                 context = stringifyNamespace(functionContext);
                                 fctx = findNamespace(functionContext)->classDef;
                                 if (!fctx->complained) {
-                                    qWarning("%s:%d: Class '%s' lacks Q_OBJECT macro\n",
-                                             qPrintable(yyFileName), yyLineNo,
-                                             qPrintable(context));
+                                    yyMsg() << qPrintable(LU::tr("Class '%1' lacks Q_OBJECT macro\n")
+                                                         .arg(context));
                                     fctx->complained = true;
                                 }
                                 goto gotctx;
@@ -1789,9 +1854,8 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                     int last = prefix.lastIndexOf(strColons);
                     QString className = prefix.mid(last == -1 ? 0 : last + 2);
                     if (!className.isEmpty() && className == functionName) {
-                        qWarning("%s::%d: It is not recommended to call tr() from within a constructor '%s::%s' ",
-                                  qPrintable(yyFileName), yyLineNo,
-                                  className.constData(), functionName.constData());
+                        yyMsg() << qPrintable(LU::tr("It is not recommended to call tr() from within a constructor '%1::%2'\n")
+                                .arg(className).arg(functionName));
                     }
 #endif
                     prefix.chop(2);
@@ -1806,9 +1870,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                             context = fctx->trQualification;
                         }
                         if (!fctx->hasTrFunctions && !fctx->complained) {
-                            qWarning("%s:%d: Class '%s' lacks Q_OBJECT macro\n",
-                                     qPrintable(yyFileName), yyLineNo,
-                                     qPrintable(context));
+                            yyMsg() << qPrintable(LU::tr("Class '%1' lacks Q_OBJECT macro\n").arg(context));
                             fctx->complained = true;
                         }
                     } else {
@@ -1820,6 +1882,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
               gotctx:
                 recordMessage(line, context, text, comment, extracomment, msgid, extra, utf8, plural);
             }
+            sourcetext.clear(); // Will have warned about that already
             extracomment.clear();
             msgid.clear();
             extra.clear();
@@ -1829,8 +1892,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
             if (!tor)
                 goto case_default;
             if (!sourcetext.isEmpty())
-                qWarning("%s:%d: //%% cannot be used with translate() / QT_TRANSLATE_NOOP(). Ignoring\n",
-                         qPrintable(yyFileName), yyLineNo);
+                yyMsg() << qPrintable(LU::tr("//% cannot be used with translate() / QT_TRANSLATE_NOOP(). Ignoring\n"));
             utf8 = (yyTok == Tok_translateUtf8);
             line = yyLineNo;
             yyTok = getToken();
@@ -1876,6 +1938,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                 }
                 recordMessage(line, context, text, comment, extracomment, msgid, extra, utf8, plural);
             }
+            sourcetext.clear(); // Will have warned about that already
             extracomment.clear();
             msgid.clear();
             extra.clear();
@@ -1884,8 +1947,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
             if (!tor)
                 goto case_default;
             if (!msgid.isEmpty())
-                qWarning("%s:%d: //= cannot be used with qtTrId() / QT_TRID_NOOP(). Ignoring\n",
-                         qPrintable(yyFileName), yyLineNo);
+                yyMsg() << qPrintable(LU::tr("//= cannot be used with qtTrId() / QT_TRID_NOOP(). Ignoring\n"));
             //utf8 = false; // Maybe use //%% or something like that
             line = yyLineNo;
             yyTok = getToken();
@@ -1952,15 +2014,13 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                     if (isspace(c))
                         continue;
                     if (c != '"') {
-                        qWarning("%s:%d: Unexpected character in meta string\n",
-                                 qPrintable(yyFileName), yyLineNo);
+                        yyMsg() << qPrintable(LU::tr("Unexpected character in meta string\n"));
                         break;
                     }
                     forever {
                         if (p >= yyWord.length()) {
                           whoops:
-                            qWarning("%s:%d: Unterminated meta string\n",
-                                     qPrintable(yyFileName), yyLineNo);
+                            yyMsg() << qPrintable(LU::tr("Unterminated meta string\n"));
                             break;
                         }
                         c = yyWord.unicode()[p++].unicode();
@@ -1994,9 +2054,14 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
                     } else {
                         context = comment.left(k);
                         comment.remove(0, k + 1);
-                        recordMessage(yyLineNo, context, QString(), comment, extracomment,
-                                      QString(), TranslatorMessage::ExtraData(), false, false);
+                        TranslatorMessage msg(
+                                transcode(context, false), QString(),
+                                transcode(comment, false), QString(),
+                                yyFileName, yyLineNo, QStringList(),
+                                TranslatorMessage::Finished, false);
+                        msg.setExtraComment(transcode(extracomment.simplified(), false));
                         extracomment.clear();
+                        tor->append(msg);
                         tor->setExtras(extra);
                         extra.clear();
                     }
@@ -2008,8 +2073,7 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
         case Tok_Arrow:
             yyTok = getToken();
             if (yyTok == Tok_tr || yyTok == Tok_trUtf8)
-                qWarning("%s:%d: Cannot invoke tr() like this\n",
-                          qPrintable(yyFileName), yyLineNo);
+                yyMsg() << qPrintable(LU::tr("Cannot invoke tr() like this\n"));
             break;
         case Tok_ColonColon:
             if (yyBraceDepth == namespaceDepths.count() && yyParenDepth == 0 && !yyTokColonSeen)
@@ -2041,9 +2105,13 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
         case Tok_Semicolon:
             prospectiveContext.clear();
             prefix.clear();
-            extracomment.clear();
-            msgid.clear();
-            extra.clear();
+            if (!sourcetext.isEmpty() || !extracomment.isEmpty() || !msgid.isEmpty() || !extra.isEmpty()) {
+                yyMsg() << qPrintable(LU::tr("Discarding unconsumed meta data\n"));
+                sourcetext.clear();
+                extracomment.clear();
+                msgid.clear();
+                extra.clear();
+            }
             yyTokColonSeen = false;
             yyTok = getToken();
             break;
@@ -2067,6 +2135,9 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
         default:
             if (!yyParenDepth)
                 prospectiveContext.clear();
+            // fallthrough
+        case Tok_Equals: // for static initializers; other cases make no difference
+        case Tok_RightBracket: // ignoring indexing; same reason
         case_default:
             yyTok = getToken();
             break;
@@ -2074,13 +2145,17 @@ void CppParser::parseInternal(ConversionData &cd, QSet<QString> &inclusions)
     }
 
     if (yyBraceDepth != 0)
-        qWarning("%s:%d: Unbalanced opening brace in C++ code"
-                  " (or abuse of the C++ preprocessor)\n",
-                  qPrintable(yyFileName), yyBraceLineNo);
+        yyMsg(yyBraceLineNo)
+            << qPrintable(LU::tr("Unbalanced opening brace in C++ code"
+                                 " (or abuse of the C++ preprocessor)\n"));
     else if (yyParenDepth != 0)
-        qWarning("%s:%d: Unbalanced opening parenthesis in C++ code"
-                 " (or abuse of the C++ preprocessor)\n",
-                 qPrintable(yyFileName), yyParenLineNo);
+        yyMsg(yyParenLineNo)
+            << qPrintable(LU::tr("Unbalanced opening parenthesis in C++ code"
+                                 " (or abuse of the C++ preprocessor)\n"));
+    else if (yyBracketDepth != 0)
+        yyMsg(yyBracketLineNo)
+            << qPrintable(LU::tr("Unbalanced opening bracket in C++ code"
+                                 " (or abuse of the C++ preprocessor)\n"));
 }
 
 const ParseResults *CppParser::recordResults(bool isHeader)
@@ -2141,8 +2216,7 @@ void loadCPP(Translator &translator, const QStringList &filenames, ConversionDat
 
         QFile file(filename);
         if (!file.open(QIODevice::ReadOnly)) {
-            cd.appendError(QString::fromLatin1("Cannot open %1: %2")
-                .arg(filename, file.errorString()));
+            cd.appendError(LU::tr("Cannot open %1: %2").arg(filename, file.errorString()));
             continue;
         }
 
@@ -2150,9 +2224,9 @@ void loadCPP(Translator &translator, const QStringList &filenames, ConversionDat
         QTextStream ts(&file);
         ts.setCodec(codec);
         ts.setAutoDetectUnicode(true);
-        if (ts.codec()->name() == "UTF-16")
-            translator.setCodecName("System");
         parser.setInput(ts, filename);
+        if (cd.m_outputCodec.isEmpty() && ts.codec()->name() == "UTF-16")
+            translator.setCodecName("System");
         Translator *tor = new Translator;
         tor->setCodecName(translator.codecName());
         parser.setTranslator(tor);
